#!/usr/bin/ruby

$KCODE = 'e'
require 'tk'
require 'socket'
require 'jukedir'

#print TkFont.families.join("\n")

#-------------------------------------------------------------------------------
#
#	Config
#
windowTitle = 'SVX HDD Jukebox System'
windowSize  = '640x480+0+0'				# '800x600+0+0'
iconify		= nil
homeDir		= '/cygdrive/d/My Documents/My Music'

titleFont	= TkFont.new(['Osaka', 24, ['bold']])
listFont	= TkFont.new(['Osaka', 12, ['bold']])
listLines	= 15
playingFont	= TkFont.new(['Osaka', 20, ['bold']])
elapsFont	= TkFont.new(['Osaka', 20, ['italic']])

#-------------------------------------------------------------------------------
#
#	Define Window
#
root = TkRoot.new {
	title		windowTitle
	geometry	windowSize
}
root.iconify if iconify

#-------------------------------------------------------------------------------
#
#	Define Title Area
#
titleArea = TkLabel.new {
	text		'album title / artist'
	font		titleFont
	fg			'white'
	bg			'black'
	anchor		'center'
}

#-------------------------------------------------------------------------------
#
#	Define Jacket Area
#
jacketImage = TkPhotoImage.new {
	file		'noimage.gif'
}
jacketArea = TkLabel.new {
	image		jacketImage
	bg			'blue'
}

#-------------------------------------------------------------------------------
#
#	Define List Area
#
songList = []
listArea = TkFrame.new {
	bg			'black'
}
(1..listLines).each {|n|
	songList[n] = TkLabel.new(listArea) {
		text		n.to_s + ". xxxxxxxx"
		font		listFont
		fg			'#606060'
		bg			'black'
		width		999
		anchor		'w'
	}
	songList[n].pack('side'=>'top')								# Framepack
}

#-------------------------------------------------------------------------------
#
#	Define Status Area
#
statusArea = TkFrame.new {
	bg		'black'
}
songName = TkLabel.new(statusArea) {
	text		'xxxxxxxxxxxxxxxx'
	font		playingFont
	fg			'white'
	bg			'black'
	anchor		'w'
}
elapsTime = TkLabel.new(statusArea) {
	text		' xx : xx '
	font		elapsFont
	fg			'white'
	bg			'black'
	anchor		'e'
}
elapsTime.pack('side'=>'right')									# Framepack
songName.pack('side'=>'left', 'fill'=>'x')

#-------------------------------------------------------------------------------
#
#	Define AboutBox
#
def aboutBox(title)
	Tk.messageBox(
		'icon'		=> 'info',
		'type'		=> 'ok',
		'title'		=> "About #{title}",
		'message'	=> <<EOF
carjuke.rb\n
version 1.00\n\n
Copyright 2004 ITLine Inc.\n
http://www.itline.jp/~svx/diary/\n
EOF
	)
end
root.bind('F1', proc{ aboutBox(windowTitle) })

#-------------------------------------------------------------------------------
#
#	Build Window
#
titleArea.pack('side'=>'top', 'fill'=>'x')
statusArea.pack('side'=>'bottom', 'fill'=>'x')
jacketArea.pack('side'=>'left', 'expand'=>true, 'fill'=>'both')
listArea.pack('side'=>'right', 'fill'=>'y')

jukeDaemon = TCPSocket.open('localhost', 2621)					# Daemon³

jukeDir = JukeDir.new(homeDir)
jukeDir.read													# ǥ쥯ȥ
jukeDir.get_resume												# ǻ

cmd = Array.new
Tk.root.bind('KeyPress', proc {|a,k,kk,n|
	print "%A:#{a} %k:#{k} %K:#{kk} %N:#{n}\n"
	cmd.push 'next'			if a == '6'
	cmd.push 'prev'			if a == '4'
	cmd.push 'replay'		if a == '0'
	cmd.push 'prev_disc'	if a == '2'
	cmd.push 'next_disc'	if a == '8'
}, "%A %k %K %N")

Thread.new(cmd) {|cmd|
	while 1
		#-----------------------------------------------------------------------
		#
		#	Display Info
		#
		titleArea.text(jukeDir.get_current_title)
		jacketImage.file(jukeDir.get_current_jacket[JukeDir::FILE])
		jukeDir.get_program_list(listLines) {|i, g, c|		# loop counter, info array, is current
			songList[i].text(g ? "#{g[JukeDir::NUM]}: #{g[JukeDir::NAME]}" : '')
			songList[i].fg(c ? 'white' : '#606060')
		}
		songName.text(jukeDir.get_current_song[JukeDir::NAME])

		#-----------------------------------------------------------------------
		#
		#	Request to Daemon
		#
		jukeDaemon.write("!#{JukeDir.to_winPath(jukeDir.get_current_song[JukeDir::FILE])}\n")
		jukeDaemon.gets

		#-----------------------------------------------------------------------
		#
		#	Loop
		#
		elapsA = Time.now; elapsB = 0	# Aϼ¬ɽBϥɽ
		while 1
			elaps = Time.now - elapsA	# elapsB
			elapsTime.text(sprintf(' %02d : %02d ', elaps / 60, elaps % 60))
			sleep 0.5; elapsB += 0.5

			if command = cmd.shift								# got command
				jukeDaemon.write("stop\n")
				jukeDaemon.gets

				if command == 'next'
					jukeDir.next
					break

				elsif command == 'prev'
					jukeDir.prev
					break

				elsif command == 'replay'
					break

				elsif command == 'next_disc'
					jukeDir.next_disc
					break

				elsif command == 'prev_disc'
					jukeDir.prev_disc
					break
				end
			end

			jukeDaemon.write("now\n")							# check playing
			unless jukeDaemon.gets =~ /ok .+/
				jukeDir.next
				break
			end
		end
	end
}

Tk.mainloop
