#!/usr/bin/ruby

require 'socket'
require './juked.conf'

#-------------------------------------------------------------------------------
#
#	Class: Player
#
class Player
	attr_reader :playing

	def initialize
		@playlist	= []
		reset
	end

	def reset
		@player_pid	= nil
		@playing	= nil
	end

	def book(filename)
		@playlist.push(filename)								# ͽꥹȤɲ
	end

	def play
		if @player_pid == nil and @playing = @playlist.shift	# ɥ뤫ͽ󤬤ʤ鳫
			@player_pid = fork {
				@playing.untaint
				exec(PlayerCommand, @playing)
			}
		end
	end

	def replay
		@playlist.unshift(@playing) if @playing					# ꥹȤ˵ͤľ
		Process.kill :TERM, @player_pid if @player_pid
	end

	def next
		Process.kill :TERM, @player_pid if @player_pid
	end

	def stop
		@playlist.clear											# ꥹȤ
		Process.kill :TERM, @player_pid if @player_pid
	end
end

#-------------------------------------------------------------------------------
#
#	Main
#
trap(:CHLD, "raise 'playing was stopped.'")
listen = TCPServer.open(PlayerPort)
socks = [listen]
player = Player.new

loop {
	begin
		nsock = select(socks)									# nsock = [reads[], writes[], excepts[]]
	rescue
		player.reset
		player.play
		next
	end

	nsock[0].each {|s|											# nsock[0] is read socket array
		if s == listen											# 饤Ȥ³
			socks.push(s.accept)
		else
			if s.eof?											# 饤ȤǤ
				s.close
				socks.delete(s)
			else
				req = s.gets									# ꥯȤ
				req.chomp!

				if req =~ /^!(.*)/
					player.book($1)
					player.play
					s.write("ok #{req}\n")

				elsif req =~ /^replay/
					player.replay
					player.play
					s.write("ok #{req}\n")

				elsif req =~ /^next/
					player.next
					player.play
					s.write("ok #{req}\n")

				elsif req =~ /^stop/
					player.stop
					s.write("ok #{req}\n")

				elsif req =~ /^now/
					if player.playing
						s.write("ok #{player.playing}\n")
					else
						s.write("ok\n")
					end
	
				else
					s.write("unknown_command #{req}\n")
				end
			end
		end
	}
}
