#!/usr/bin/ruby

#-------------------------------------------------------------------------------
#
#	Class: JukeDir
#
class JukeDir

	#---------------------------------------------------------------------------
	#
	#	Constructor
	#
	def initialize(topDir)
		@topDir			= topDir
		@crDir			= 0
		@crFile			= 0
		@fileType		= '(mp3|mpeg|mpg)'
		@jacketFile		= 'jacket.gif'
		@listFile		= 'songs.lst'
		@noImageFile	= "#{@topDir}/noimage.gif"
		@resumeFile		= "#{@topDir}/resume.txt"
	end

	#---------------------------------------------------------------------------
	#
	#	Read dirctory and Store
	#
	NUM  = 0; NAME = 1; FILE = 2
	def read
		@dirs = Array.new
		Dir.foreach(@topDir) {|n|
			if n !~ /\./ and FileTest.directory?("#{@topDir}/#{n}")
				temp = Hash.new

				temp['jacket'] = FileTest.exist?("#{@topDir}/#{n}/#{@jacketFile}") ?
					[0, 'jacket', "#{@topDir}/#{n}/#{@jacketFile}"] : [0, 'no jacket', @noImageFile]

				temp['files'] = Array.new
				if FileTest.exist?("#{@topDir}/#{n}/song.lst")		# ꥹȥե뤢
					files = open("#{@topDir}/#{n}/song.lst").readlines
					temp['title'] = files.shift.chomp
					i = 0; files.each {|nn|
						temp['files'].push([i += 1, $2, "#{@topDir}/#{n}/#{$1}"]) if nn =~ /(.*?),(.*)/
					}
				else												# ꥹȥեʤ
					temp['title'] = n
					i = 0; Dir.foreach("#{@topDir}/#{n}") {|nn|
						temp['files'].push([i += 1, $1, "#{@topDir}/#{n}/#{nn}"]) if nn =~ /(.*)\.#{@fileType}$/i
					}
				end

				@dirs.push temp if temp['files'].length != 0
			end
		}
	end

	#---------------------------------------------------------------------------
	#
	#	Access Resume file
	#
	def get_resume
		if FileTest.exist?(@resumeFile)
			info = open(@resumeFile).readlines
			if info[0] =~ /(\d+),(\d+)/
				@crDir  = $1.to_i
				@crFile = $2.to_i
			end
		end
	end

	def set_resume
		open(@resumeFile, 'w') {|info|
			info.write "#{@crDir},#{@crFile}\n"
		}
		[@crDir, @crFile]
	end

	#---------------------------------------------------------------------------
	#
	#	Get Current Info
	#
	def get_current_jacket
		@dirs[@crDir]['jacket']
	end

	def get_current_title
		@dirs[@crDir]['title']
	end

	def get_current_song
		@dirs[@crDir]['files'][@crFile]
	end

	#---------------------------------------------------------------------------
	#
	#	Change Current Song / Disc
	#
	def next
		@crFile += 1
		while @dirs[@crDir]['files'].length <= @crFile
			@crFile = 0
			@crDir = (@crDir + 1) % @dirs.length
		end
		set_resume
	end

	def prev
		@crFile -= 1
		while @crFile < 0
			@crDir = (@crDir - 1) % @dirs.length
			@crFile = @dirs[@crDir]['files'].length - 1
		end
		set_resume
	end

	def next_disc
		@crFile = 0
		@crDir = (@crDir + 1) % @dirs.length
		set_resume
	end

	def prev_disc
		@crFile = 0
		@crDir = (@crDir - 1) % @dirs.length
		set_resume
	end

	#---------------------------------------------------------------------------
	#
	#	Get Program list for Display
	#
	def get_program_list(showLines)
		mid = ((showLines - 0.1) / 2).to_i
		top = @crFile - mid
		btm = top + showLines
		dif = btm - @dirs[@crDir]['files'].length
		begin top -= dif; btm -= dif; end if dif > 0
		begin btm -= top; top -= top; end if top < 0
		i = 0; (top..btm - 1).each {|n|
			if @dirs[@crDir]['files'][n]
				yield(i += 1, @dirs[@crDir]['files'][n], @crFile == n)
			else
				yield(i += 1, nil, nil)
			end
		}
	end

	#---------------------------------------------------------------------------
	#
	#	Convert unix path to windows path (class method)
	#
	def self.to_winPath(unixPath)
		"#{$1}:#{$2}".gsub(/\//, '\\') if unixPath =~ /\/cygdrive\/([a-z])(\/.*)/i
	end
end
